#!/usr/bin/python
# -*- coding: UTF-8 -*-
#   02.12.2013

# Testdatei:
# führt Remote At-Commands mit dem Coordinator (Id = 0) aus und muss immer funktionieren,
# sofern ein Coordinator präsent ist
# auch wenn die Adressliste falsche Einträge enthält
#
# Die empfangenen Daten werden von self.show() bearbeitet.
#
# durch Austausch der Stationsnummer kann die Verbindung zu anderen Station
# aufgenommen werden, aber deren Seriennummer muss dazu in der xbee.cfg
# eingetragen sein.
# hier muss dev geändert werden -> self.X.sendRemoteAtCmd(dev = 0, atcmd = cmd)


import atexit
import time
import xbee
import SerPort
import xbee_help

class App(object):
    def __init__(self):
        # hier wird eine Serielle Schnittstelle geöffnet,
        self.ser = SerPort.open()
        self.start()

    def receive(self, station, frame):
        """ Wird als Callback-Funktion an das Objekt Xbee() übergeben und
        liefert folgende Daten zurück:
        - station = Index des Absenders in der Senderliste,
        - frame = komplettes Frame als Bytefolge (ohne abschließende Prüfsumme).
        Die Frames 0x88, 0x8B und 0x97 werden im Normalfall vom rxHandler
        interpretiert und kommen hier nicht an !
        """
        # die Daten zur Interpretation an xbee_help weiterreichen
        xbee_help.x_print(frame)


    def start(self):
        self.X = xbee.Xbee(ser = self.ser, callback = self.receive,
                filename = None, logToFile = False, logToTTY = False, readRSSI = False)
        atexit.register(self.X.exit)

        # ruft mehrfach lokale AT-Commands auf
        # das muss immer funktionieren, auf wenn die Stationslist nicht aktuell ist
        # und wenn keine Gegenstelle vorhanden ist !
        j = 5
        cmdlist = ("MY", "SH", "SL", "DB", "D1")
        try:
            while j:
                j -= 1
                cmd = cmdlist[j]
                self.X.sendRemoteAtCmd(dev = 0, atcmd = cmd, rxHandler = self.show)
                time.sleep(1)

        except KeyboardInterrupt:
            self.X.exit()
            exit()

        time.sleep(1)
        # die Stationliste ausgeben, auf wenn das Loggen deaktiviert ist
        self.X.config(logToTTY = True)
        self.X.X_adr.printStationList()
        self.X.exit()
        exit()


    def show(self, station, data):
        """ Zeigt die empfangenen Daten an """
        s = ""
        for i in data:
            s += " {:02X}".format(i)
        print("Station: {}, Data: {}".format(station, s))

if __name__ == "__main__":
    App()


# --- [ eof ] ------------------------------------------------------------------