#!/usr/bin/python
# -*- coding: UTF-8 -*-
#   02.12.2013

# Hier werden die lokalen AtCommandes "SH" und "SL" gesendet und
# der Funktion readMyAdr zur Auswertung übergeben.
# readMyAdr ermittelt die eigene 64-Bit Seriennummer und holt sich damit
# aus der Stationsliste den zugehörigen Index und Namen.
# In der Stationsliste wird die eigene Station durch *** markiert

import atexit
import time
import xbee
import SerPort
import xbee_help
import xbee_jobs

class App(object):
    def __init__(self):
        self.ser = SerPort.open()
        self.adr64bit = b'\00\00\00\00\00\00\00\00'
        self.name  = "?"
        self.Xadr  = None
        self.start()


    def receive(self, station, frame):
        """ Wird als Callback-Funktion an das Objekt Xbee() übergeben und
        liefert folgende Daten zurück:
        - station = Index des Absenders in der Senderliste,
        - frame = komplettes Frame als Bytefolge (ohne abschließende Prüfsumme).
        Die Frames 0x88, 0x8B und 0x97 werden im Normalfall vom rxHandler
        interpretiert und kommen hier nicht an !
        """
        # die Daten zur Interpretation an xbee_help weiterreichen
        xbee_help.x_print(frame)


    def doNix(self, station, frame):
        """ dieser rxHandler mach wirklich nichts
            wird verwendet, damit ein Remote At Command versendet werden kann,
            dessen Rückmeldung nicht von Interesse ist.
            Denn nur die Empfangsstärke des Signals soll ermittelt werden.
        """
        pass


    def readMyAdr(self, station, frame):
        """ Liest mittels AT-Commands die eigene Seriennummer aus.
            Fügt die Antworten auf die lokalen AT-Commands SH / SL
            zu einer 64-Bit Adresse,
            prüft deren Existenz in der Stationsliste und
            gibt den Index der Station zurück,
            ermittelt über den Index den Namen der Station.
            Anschießend wird self.adr_self_idx in Xbee_adr()
            mit dem Index belegt, so dass die angeschlossene
            Station in der Stationsliste markiert werden kann. """

        data = frame[-4:]           # die Daten stehen in den letzten 4 Bytes
        # die Adresse ist als Bytestring der Länge 8 vorbelegt.
        if frame[5:7] == bytes(("SH").encode('ascii')):
            self.adr64bit = data + self.adr64bit[4:]

        elif frame[5:7] == bytes(("SL").encode('ascii')):
            self.adr64bit = self.adr64bit[:4] + data

        tmp = self.X.X_adr.getStation(adr64 = self.adr64bit)
        idx = tmp[0]
        self.name = tmp[3]

        self.X.X_adr.setSelfIndex(idx)

        adr = ""
        for i in self.adr64bit:
            adr += " {:>02X}".format(i)
        print("readMyAdr() -> 64-Bit Adresse:{}, Index: {}, mein Name ist: {}\n".
                                                format(adr, idx, self.name))

        # jetzt wird noch die 16-Bit Netzwerkadresse eingelesen
        # und dabei die Stationsliste aktualisiert
        # dazu wird ein beliebiges RemoteAtCmd an die eigende Adresse geschickt
        # im empfangenen Frame ist die 16-Bit Adresse enthalten.
        # Ausserdem wird die Empfangsstärke gemessen - in diesem Fall -00db
        if idx > -1:
            self.X.sendRemoteAtCmd(idx, "SH", rxHandler = self.doNix)


    def start(self):
        self.X = xbee.Xbee(ser = self.ser, callback = self.receive,
                                                        readRSSI = True)
        atexit.register(self.X.exit)

        # sendet die lokalen AT-Commands "SL" und "SH" und übergibt die Rückmeldung an
        # die Funktion readMyAdr()

        self.X.sendLocalAtCmd(atcmd = "SL", rxHandler = self.readMyAdr)
        self.X.sendLocalAtCmd(atcmd = "SH", rxHandler = self.readMyAdr)

        # etwas abwarten, damit die Rückmeldung eintreffen kann
        time.sleep(.5)

        # die Stationliste ausgeben, auf wenn das Loggen deaktiviert ist
        self.X.X_adr.config(logToTTY = True)
        self.X.X_adr.printStationList()
        self.X.exit()
        exit()


if __name__ == "__main__":
    App()


# --- [ eof ] ------------------------------------------------------------------