#!/usr/bin/python
# -*- coding: UTF-8 -*-
#  02.12.2013

# hier werden direkt nacheinander ohne Pause 27 Frames mit unterschiedlichen
# Inhalten und unterschiedlichen rxHandlern abgeschickt.
# Station 0 ist der Coordinator,
# wenn das angeschlossene Xbee selbst der Coordinator ist, werden alle Frames
# an die eigene Adresse geschickt.
# Daraus mag sich der nachfolgende Fehler erklären:
# wenn beim Senden keine Warteschleife sleep(.1) vorhanden ist,
# denn geht immer genau ein DatenFrame 0x8B mit der Id 2 verloren.
#
# Werden Remote und DataFrames an externe Stationen gesendet, dann tritt der
# Fehler nicht auf.

# Die Ausgabe zeigt an, welche Funktion welches Frame mit welchem Kommando
# bearbeitet hat.

# Werden mehr als ca. 9 Frames versandt, dann läuft offensichtlich der interne
# Puffer des Xbee zu - serialwin32.py meldet einen timeout Fehler.
#
# Nur durch Abfrage von CTS (readCTS = True) läßt sich dieses Problem vermeiden.
# Allerdings muss der CTS-Ausgang des Xbee (Pin 12) auch angeschlossen sein,
# sonst tut sich gar nichs mehr beim Senden.

import atexit
import time
import xbee
import SerPort
import xbee_help

class App(object):
    def __init__(self):
        self.ser = SerPort.open()
        self.adr64bit = bytes(8)
        self.name  = "?"
        self.start()


    def receive(self, station, frame):
        """ Wird als Callback-Funktion an das Objekt Xbee() übergeben und
        liefert folgende Daten zurück:
        - station = Index des Absenders in der Senderliste,
        - frame = komplettes Frame als Bytefolge (ohne abschließende Prüfsumme).
        Die Frames 0x88, 0x8B und 0x97 werden im Normalfall vom rxHandler
        interpretiert und kommen hier nicht an !
        """
        # die Ausgabe der Frame0x90 soll unterdrückt werden
        #xbee_help.x_print(frame)
        pass


    def func97(self, station, frame):
        """ behandelt alle Frames 0x97 """
        if frame[0] != 0x7E:
            print("Func97():  Error: {}, FrameId: {}, no answer from Station: {}". format(
                                    "{:02X}".format(frame[0]),frame[1], frame[3]))
            return

        if len(frame) > 18:
            data = frame[18:]
        else:
            data = ""
        print("Func97(): empfange FrameId: {}, Frametyp: {}, AtCmd: {}, Status: {}, Data: {}".format(
                                    "{:3}".format(frame[4]),
                                    "{:02X}".format(frame[3]),
                                     frame[15:17].decode(),
                                     frame[17],
                                     data  ))

    def func88(self, station, frame):
        """ Behandelt alle Frames 0x88 """
        if frame[0] != 0x7E:
            print("Func88(): Error: {}, FrameId: {}, no answer from Station: {}". format(
                                    "{:02X}".format(frame[0]),frame[1], frame[3]))
            return

        if len(frame) > 7:
            data = frame[8:]
        else:
            data = ""
        print("Func88(): empfange FrameId: {}, Frametyp: {}, AtCmd: {}, Status: {}, Data: {}".format(
                                    "{:3}".format(frame[4]),
                                    "{:02X}".format(frame[3]),
                                    frame[5:7].decode(),
                                    frame[7],
                                    data ) )

    def func8B(self, station, frame):
        """ Behandelt alle Frames 0x8B """
        if frame[0] != 0x7E:
            print("Func8B(): Error: {}, FrameId: {}, no answer from Station: {}". format(
                        "{:02X}".format(frame[0]), frame[1], frame[3]))
            return

        print("Func8B(): empfange FrameId: {}, Frametyp: {}, Status: {} {}".format(
                                    "{:3}".format(frame[4]),
                                    "{:02X}".format(frame[3]),
                                    "{:02X}".format(frame[8]) ,
                                    "{:02X}".format(frame[9]) ) )


    def start(self):
        self.X = xbee.Xbee(ser = self.ser, callback = self.receive,
                                   readRSSI = False, readCTS = True)
        atexit.register(self.X.exit)
        #time.sleep(.5)

        try:
            for i in ("SL", "SH", "MY", "D0", "D1", "D3", "D4", "D5", "D6" ):
                self.X.sendLocalAtCmd(atcmd = i, rxHandler = self.func88)
                self.X.sendRemoteAtCmd(atcmd = i, dev = 0, rxHandler = self.func97)
                self.X.sendDataFrame( dev = 0, data = [1,2,3], rxHandler = self.func8B)

                # ohne sleep(0.1) geht ein DataFrame 0x8B verloren (FrameId 2)
                # allerdings nur dann, wenn alle Sendungen vom Coordinator an
                # den Coordinator gehen - rein praktisch ist das sinnfrei ...
                time.sleep(.1)

        except KeyboardInterrupt:
            self.X.printStationList()
            self.X.exit()
            exit()

        print("waiting ...")
        time.sleep(3)

        self.X.X_jobs.cleanJobs(2)  # Jobs löschen, die älter als 2 Sekunden sind

        print("Gesendet:  {}, quittiert: {}, empfangene Datenframes: {}".format(
                self.X.cnt_tx, self.X.cnt_rx, self.X.cnt_data))

        self.X.exit()
        exit()


if __name__ == "__main__":
    App()


# --- [ eof ] ------------------------------------------------------------------