#!/usr/bin/python
# -*- coding: UTF-8 -*-
#   28.11.2013

# Hier das Grundgerüst einer Anwendung dargestellt:
# 1.) Es wird eine Instanz von Xbee() angelegt.
# 2.) Eine Endlosschleife beginnt und startet mit einem sleep.
# 3.) Nach Ablauf des Sleep
# 4.) - werden Unterprogramme ausgeführt, die zwischenzeitlich in eine ToDo-Liste
#       eingefügt wurden.
# 5.) - werden andere Unterprogramme in regelmäßigen Zeitabschnitten ausgeführt.

#     - alle 30 Sekunden wird ein Frame an Station 5 geschickt (die offline ist)
#       vor dem Absenden wird geprüft, ob eine Antworten der Station aussteht.
#       Wenn ja, dann wird kein Frame versandt
#       - alle 60 Sekunden werden alle Jobs gelöscht, die älter als 180 Sekunden sind
#       - alle 180 Sekunden wird eine Stationsliste ausgegeben.

#    - verkürzt man die Wartezeit zwischen den Sendungen auf 1 Sekunde, dann wird jede
#      2. Sendung nicht gesendet, da die Fehlermeldung noch nicht eingetroffen ist.
#    - __doNothing(): gibt einen FrameErr aus
#       (die Gegenstelle ist nicht erreichbar, daher ist der cmdstatus > 0 !)

import atexit
import time
import xbee
import SerPort
import xbee_help


class App(object):
    def __init__(self):
        self.ser = SerPort.open()
        self.lock = False
        self.start()


    def receive(self, station, frame):
        """ Wird als Callback-Funktion an das Objekt Xbee() übergeben und
        liefert folgende Daten zurück:
        - station = Index des Absenders in der Senderliste,
        - frame = komplettes Frame als Bytefolge (ohne abschließende Prüfsumme).
        Die Frames 0x88, 0x8B und 0x97 werden im Normalfall vom rxHandler
        interpretiert und kommen hier nicht an !
        """
        # die Daten zur Interpretation an xbee_help weiterreichen
        xbee_help.x_print(frame)


    def start(self):
        """ Die Hauptschleife der Applikation """
        self.X = xbee.Xbee(ser = self.ser, callback = self.receive,
                                logToTTY = False, readRSSI = True, readCTS = True)
        atexit.register(self.X.exit)

        self.X.readMyAdr()          # ermittelt die eigene Adresse
        time.sleep(.5)              # etwas warten, bevor das Ergebnis gelesen wird
        print("\nMeine Seriennummer: {}, Mein Name: {}, Meine Id: {}\n". format(
                self.X.adr64bit, self.X.name, self.X.id))

        lasttime = int(time.time())

        try:
            while True:
                time.sleep(.1)

                # in regelmäßigen Abständen (1 Sekunde) Unterprogramme ausführen
                if int(time.time()) > lasttime:
                    lasttime = int(time.time())
                    self.cronjob(lasttime)  # die aktuelle Zeit wird übergeben

        except KeyboardInterrupt:
            self.X.exit()
            exit()

        self.X.exit()
        exit()


    def cronjob(self, t):
        """ führt Jobs in immer gleichen Zeitintervallen aus.
            Wird jede Sekunde 1-mal aufgerufen und prüft,
            ob es Zeit ist, einen Job zur Ausführung zu bringen.
        """
        # alle 10 Sekunden gemäß interner Uhr
##        if t % 10 == 0:
##            pass
        # eine halbe Minute (+ 5 Sekunden) gemäß interner Uhr
        #if t % 30 == 5:
        if True:    # ist immer war, wird daher im 1 Sekundenabstand ausgeführt
            x = self.X.getJobsByStation(5)
            if (x[0] == 0):          # wenn kein Job in der Warteschlange ist
                self.X.sendRemoteAtCmd(dev = 5, atcmd = "MY")
            else:
                print("Empfänger nicht bereit, Sendung wird verschoben", x)

        # eine volle Minute (+ 10 Sekunden) gemäß interner Uhr
        # die Jobliste ausmisten, die älter als 180 Sekunden sind ...
        if t % 60 == 10:
            self.X.cleanJobs()

        # eine volle Minute (+ 15 Sekunden) gemäß interner Uhr
##        if t % 60 == 15:
##            pass

        # alle 3 Minuten + 17 Sekunden gemäß interner Uhr
        if t % 180 == 17:
            self.X.config(logToTTY = True)
            self.X.X_adr.printStationList()
            self.X.X_jobs.printJobs()
            print("Gesendet:  {}, quittiert: {}, Übertragungsfehler: {}, "\
                    "empfangene Datenframes: {}".format(
                    self.X.cnt_tx, self.X.cnt_rx, self.X.cnt_err, self.X.cnt_data))
            self.X.config(logToTTY = False)

        # volle Stunde (+3 Sekunden) gemäß interner Uhr
##        if t % 3600 == 3:
##            pass


    def todo(self):
        # Alle Jobs, die das Hauptprogramm ausführen soll,
        # werden in einer ToDoListe abgelegt.
        # Hier wird immer nur der erste Datensatz ausgeführt und dann gelöscht.
        # Damit rücken eventuell nachfolgende Jobs an den Anfang der Liste
        # und werden nach dem nächsten Sleep ausgeführt.

        # Action = 1:
        # - Damit können AtCmd's ausgeführt oder Datenframes verschickt werden
        # - Format: Action, Frame, Zieldevice, AtCmd, Datenliste

        #print(self.todolist[0])

        if self.todolist[0][0] == 1:
            if self.todolist[0][1] == 0x08:     # Local AT Command
                self.X.sendLocalAtCmd(atcmd = self.todolist[0][3],
                                        data = self.todolist[0][4])

            elif self.todolist[0][1] == 0x10:   # DataFrame
                self.X.sendDataFrame(dev = self.todolist[0][2],
                                        data = self.todolist[0][4] )

            elif self.todolist[0][1] == 0x17:   # Remote AT Command
                self.X.sendRemoteAtCmd(dev = self.todolist[0][2],
                                        atcmd = self.todolist[0][3],
                                        data = self.todolist[0][4])
            else:
                print (self.todolist[0], len(self.todolist))

            while self.lock == True:
                time.sleep(.01)
            self.lock = True
            del self.todolist[0]
            self.lock = False

if __name__ == "__main__":
    App()


# --- [ eof ] ------------------------------------------------------------------