#!/usr/bin/python
# -*- coding: UTF-8 -*-
#  02.12.2013

# Hier das Grundgerüst einer Anwendung dargestellt:
# 1.) Es wird eine Instanz von Xbee() angelegt.
# 2.) Eine Endlosschleife beginnt und startet mit einem sleep.
# 3.) Nach Ablauf des Sleep
# 4.) - werden Unterprogramme ausgeführt, die in der cronliste stehen
#       und deren Zeitpunkt zur Ausführung gekommen ist.
# 5.) - werden andere Unterprogramme einmalig ausgeführt, die in der
#       timelist stehen.
#     die todolist ist überflüssig, die Aktionen können ohne diesen Umweg
#     direkt verschickt werden.


import atexit
import time
import xbee
import SerPort
import xbee_help
import xbee_frame

class App(object):
    def __init__(self):
        self.ser = SerPort.open()
        self.lock = False
        self.cronlist = []
        self.timelist = []
        self.__start()


    def addTimeList(self, data):
        """ Fügt einen Job in die timelist ein,
            dabei wird die angegebene Verzögerungszeit zur aktuellen
            Zeit addiert.
        """
        if type(data) == tuple:     # Falls ein Tuple übergeben wurde
            data = list(data)
        data[0] += int(time.time())
        while self.lock:
            time.sleep(.01)
        self.lock = True
        self.timelist.append(data)
        self.lock = False


    def addCronList(self, data):
        """ Fügt einen Job unverändert in die cronlist ein """
        while self.lock:
            time.sleep(.01)
        self.lock = True

        self.cronlist.append(data)
        self.lock = False


    def __receive(self, station, frame):
        """ Wird als Callback-Funktion an das Objekt Xbee() übergeben und
        liefert folgende Daten zurück:
        - station = Index des Absenders in der Senderliste,
        - frame = komplettes Frame als Bytefolge (ohne abschließende Prüfsumme).
        Die Frames 0x88, 0x8B und 0x97 werden im Normalfall vom rxHandler
        interpretiert und kommen hier nicht an !
        """
        # bei Bedarf kann man die Daten zur Interpretation an xbee_help weiterreichen
        #xbee_help.x_print(frame)
        self.X_frame.check(station, frame)


    def __start(self):
        """ Die Hauptschleife der Applikation """
        self.X = xbee.Xbee(ser = self.ser, callback = self.__receive,
                                logToTTY = True, readRSSI = True, readCTS = True)
        atexit.register(self.X.exit)

        self.X_frame = xbee_frame.Xbee_frame(main = self, xbee = self.X)

        self.X.readMyAdr()          # ermittelt die eigene 16-Bit Adresse

        # sendet alle 5 Sekunden ein DataFrame an den Coordinator
#        self.addCronList (( 5, 0, self.X.sendDataFrame, (0, (0x56, 1, 12, 20), self.__testFunc) ))

        # sendet alle 5 Sekunden ein At-Cmd an den Coordinator
#        self.addCronList (( 5, 0, self.X.sendRemoteAtCmd, (0, "MY", [], self.__testFunc) ))

        # sendet alle 10 Sekunden ein Lokales At-Cmd
#        self.addCronList (( 10, 0, self.X.sendLocalAtCmd, ("MY", [], self.__testFunc) ))

        # Sendet nach 5 Sekunden einmalig ein DataFrame an den Coordinator
#        self.addTimeList( [ 5, self.X.sendDataFrame, (0, (0x56, 1, 12, 20), self.__testFunc) ])

        # sendet nach 5 Sekunden einmalig ein At-Cmd an den Coordinator
#        self.addTimeList (( 5, self.X.sendRemoteAtCmd, (0, "MY", [], self.__testFunc) ))

        # sendet nach 5 Sekunden einmalig ein lokales At-Cmd
        self.addTimeList (( 5, self.X.sendLocalAtCmd, ("MY", [], self.__testFunc) ))

        # eine Funktion ohne Parameter ist zulässig ...
        self.addTimeList (( 10, self.__testFunc ))

        # eine Funktion mit Parameter natürlich auch ...
        # aber wenn die Liste nur ein einziges Interable enthält, dann gehts schief
        # ("ABCD") wird in die 4 Parameter "A", "B", "C" und "D" aufgelöst
        # ("ABCD",) hat dagegen Erfolg.
        #self.addTimeList (( 11, self.__testFunc, ("ABCD",) ))
        #self.addCronList (( 6, 2, self.__testFunc, ("XYZ",) ))
        #self.addCronList (( 6, 3, self.__testFunc  ))

        lasttime = int(time.time())
        try:
            while True:
                time.sleep(.1)
                actualtime = int(time.time())

                # in regelmäßigen Abständen (1 Sekunde) Unterprogramme ausführen
                if actualtime > lasttime:
                    lasttime = actualtime

                    if len(self.cronlist):
                        self.__cronJob(lasttime)

                    if len(self.timelist):
                        self.__timeJob(lasttime)

        except KeyboardInterrupt:
            self.X.exit()
            exit()

        self.X.exit()
        exit()


    def __cronJob(self, t):
        """ führt Jobs in immer gleichen Zeitintervallen aus.
            Wird jede Sekunde 1 mal aufgerufen und prüft,
            ob es Zeit ist, einen Job zur Ausführung zu bringen.
            Dazu wird die Systemzeit (in Sekunden) % Modulo berechnet und mit
            dem "Rest" verglichen.
            int(time.time()) % 60 == 0 -> volle Minute der Systemzeit
            Die Parameter der function werden als Tuple übergeben !
            Da Positionsparamatern gearbeitet, muss die Reihenfolge unbedingt
            eingehalten werden.
            Wird ein rxHandler übergeben, dann muss auch die datenliste vorhanden
            sein, ggf. als leere Liste !
                        0       1     2         3
            Frame 0x08: modulo, rest, function, (atcmd[, datenliste[, rxHandler]]
            Frame 0x10: modulo, rest, function, (device, datenliste[, rxHandler]
            Frame 0x17: modulo, rest, function, (device, atcmd[, datenliste[, rxHandler]])
        """

        for i in range(len(self.cronlist)):
            if ((t % self.cronlist[i][0]) == self.cronlist[i][1]):
                if len(self.cronlist[i]) > 3:   # eine Parameterliste ist verfügbar
                    self.cronlist[i][2](*self.cronlist[i][3])
                else:                           # eine Funktion ohne Parameterliste
                    self.cronlist[i][2]()


    def __timeJob(self, t):
        """ Jobs, die zu einem bestimmten Zeitpunkt einmalig ausgeführt werden sollen.
            Die Parameter der function werden als Tuple übergeben !
            Da Positionsparamatern gearbeitet, muss die Reihenfolge unbedingt
            eingehalten werden.
            Wird ein rxHandler übergeben, dann muss auch die datenliste vorhanden
            sein, ggf. als leere Liste !
                        0     1          2
            Frame 0x08: time, function, (atcmd[, datenliste[, rxHandler]] )
            Frame 0x10: time, function, (device, datenliste[, rxHandler] )
            Frame 0x17: time, function, (device, atcmd[, datenliste[, rxHandler]] )
        """
        while True:
            d = -1
            for i in range(len(self.timelist)):
                if t >= self.timelist[i][0]:
                    d = i
                    if len(self.timelist[i]) > 2:   # eine Parameterliste ist verfügbar
                        self.timelist[i][1](*self.timelist[i][2])
                    else:                           # eine Funktion ohne Parameterliste
                        self.timelist[i][1]()

            if d == -1:             # dann wurde kein passender Eintrag gefunden
                break

            else:                   # den Eintrag löschen
                while self.lock == True:
                    time.sleep(.01)
                self.lock = True
                del self.timelist[d]
                self.lock = False


    def __testFunc(self, dev = None, t = None):
        print("testFunc(): ", dev, t)

if __name__ == "__main__":
    App()


# --- [ eof ] ------------------------------------------------------------------