# -*- coding: UTF-8 -*-
#       10.11.2013
# --- [ xbee_frame.py ] ---------------------------------------------------------
#
# Zuständig für die Bearbeitung der eingegangenen Frames
import time

class Xbee_frame(object):
    def __init__(self, main, xbee):
        self.main = main
        self.X = xbee

        self.d = {
            0x88: self.__frame0x88,
            0x8B: self.__frame0x8B,
            0x90: self.__frame0x90,
            0x91: self.__frame0x91,
            0x92: self.__frame0x92,
            0x94: self.__frame0x94,
            0x95: self.__frame0x95,
            0x97: self.__frame0x97,
            0xA0: self.__frame0xA0,
            0xA1: self.__frame0xA1,
            0xA3: self.__frame0xA3
            }


    def check(self, station, frame):
        """ verteilt die Daten auf die zuständigen Routinen """
        global framedata
        framedata = ""
        for i in frame:
            framedata += "{:02X} ".format(i)

        func = self.d.get(frame[3], self.__no_frame)
        return func(station, frame)

    # ------------------------------------------------------------------------------

    def __no_frame(self, station, frame):
        print("Ohne Zuordnung: ", station, frame)

    def __frame0x88(self, station, frame):
        """ AT Command Response """
        print("Sorry, das hätte nicht passieren dürfen !")
        print("0x88 -> ", station, framedata)

    def __frame0x8A(self, station, frame):
        """ Modem Status """
        print("0x8A -> ", station, framedata)

    def __frame0x8B(self, station, frame):
        """ ZigBee Transmit Status """
        print("Sorry, das hätte nicht passieren dürfen !")
        print("0x8B -> ", station, framedata)

    def __frame0x90(self, station, frame):
        """ ZigBee Receive Packet """
        data = frame[15:]
        data_len = len(data)
        if data_len == 0:
            return

##        print("0x90 -> ", station, tag, framedata)
##        print(time.time())
##        return

        tag = data[0]

        if tag == 11:
            print("# {} joined, {} Adressen". format(station, data[1]))

        if tag == 41:
            rH = (data[1] * 256 + data[2]) / 10
            T1 = (data[3] * 256 + data[4])
            if T1 > 2**15:
                T1 = -2**16 - T1
            T1 /= 10
            L  = data[5] * 256 + data[6]
            T2 = (data[7] * 256 + data[8])
            if T2 >= 2**15:
                T2 = -2**16 + T2
            T2 /= 10

            T3 = (data[9] * 256 + data[10])
            if T3 >= 2**15:
                T3 = -2**16 + T3
            T3 /= 10
            T4 = (data[11] * 256 + data[12]) / 10

            print("RH:  {}%, T1: {}°, L: {}, T2: {}°, T3: {}°, T4: {}°".format(
                    "{:4}".format(rH),
                    "{:4}".format(T1),
                    "{:5}".format(L),
                    "{:4}".format(T2),
                    "{:4}".format(T3),
                    "{:4}".format(T4) ) )

        if tag == 61:
            # Ein Bewegungsmelder hat Alarm ausgelöst
            # Ein Frame 0x10 an den Absender schicken, der ein
            # akustisches Signal an der Station auslöst.

            # wird ohne Umweg über eine Liste direkt ausgeführt
            self.X.sendDataFrame( station, (0x56, 2, 200, 100, 1) )
            # wird mit angegebener Verzögerung ausgeführt
            # Format( (Verzögerung in Sekunden, Funktion, (Parameterliste) ) )
            self.main.addTimeList( (12, self.X.sendDataFrame, (station, (0x56, 2, 200, 100, 3) ) ) )
            self.main.addTimeList( (20, self.X.sendDataFrame, (station, (0x56, 2, 200, 100, 2) ) ) )
            self.main.addTimeList( (28, self.X.sendDataFrame, (station, (0x56, 2, 200, 100, 1) ) ) )

        if tag == 62:
            # Ein Bewegungsmelder hat den Alarm zurückgenommen
            # Ein Frame 0x10 an den Absender schicken, der ein
            # akustisches Signal an der Station auslöst.
            self.X.sendDataFrame( station, (0x56, 1, 12, 19) )

    def __frame0x91(self, station, frame):
        """ ZigBee Explicit Rx Indicator """
        print("0x91 -> ",station, frame)

    def __frame0x92(self, station, frame):
        """ ZigBee IO Data Sample Rx Indicator """
        print("0x92 -> ",station, frame)

    def __frame0x94(self, station, frame):
        """ ZigBee Sensor Read Indicator """
        print("0x94 -> ", station, frame)

    def __frame0x95(self, station, frame):
        """ Node Identifaction Indicator """
        print("0x95 -> ",station, frame)

    def __frame0x97(self, station, frame):
        """ Remote Command Response """
        print("Sorry, das hätte nicht passieren dürfen !")
        print("0x97 -> ",station, framedata)

    def __frame0xA0(self, station, frame):
        """ Over-the-air Firmware Update Status """
        print("0xA0 -> ",station, frame)

    def __frame0xA1(self, station, frame):
        """ Route Record Indicator """
        print("0xA1 -> ",station, frame)

    def __frame0xA3(self, station, frame):
        """ Many-to-One Route Request Indicator """
        print("0xA3 -> ",station, frame)

##    d = {
##        0x88: self.__frame0x88,
##        0x8B: self.__frame0x8B,
##        0x90: self.__frame0x90,
##        0x91: self.__frame0x91,
##        0x92: self.__frame0x92,
##        0x94: self.__frame0x94,
##        0x95: self.__frame0x95,
##        0x97: self.__frame0x97,
##        0xA0: self.__frame0xA0,
##        0xA1: self.__frame0xA1,
##        0xA3: self.__frame0xA3
##        }
    # --- [ eof ] ------------------------------------------------------------------