#!/usr/bin/python
# -*- coding: UTF-8 -*-

import tkinter as Tk

class Gui(object):
    def __init__(self, parent):
        frm = Tk.LabelFrame(parent, text = "Raumliste", height = 200)
        frm.grid(column = 0, row = 0, padx = 10, pady = 10)
                      # id, Name,   col, row
        self.rooms =(   (3, "Raum 1", 0, 0),
                        (6, "Raum 2", 4, 0),
                        (1, "Raum 3", 0, 1) )
        self.roomlist = []

        for i in range(len(self.rooms)):
            self.roomlist.append(
                            Room( frm,
                            id   = self.rooms[i][0],
                            name = self.rooms[i][1],
                            col  = self.rooms[i][2],
                            row  = self.rooms[i][3] ) )



    def setCallback(self, callback):
        """ Weist eine Callback-Funktion zu, genaugenommen wird
            sie an die Räume weitergereicht """
#        print("gui.setCallback", callback)
        for i in self.roomlist:
            i.setCallback(callback)

    def cb(self, station, data):
        """ Diese Methode wird als Callback ausgeführt,
            wenn ein Frame 0x90 empfangen wird """

#        print("gui.callback()", station, data)
        if data[0] == 45:
            l = data[1] * 256 + data[2]

            for i in range(len(self.rooms)):
                if self.rooms[i][0] == station:
                    self.varlist[i].setLight(l)
        elif data[0] == 61:
            for i in range(len(self.rooms)):
                if self.rooms[i][0] == station:
                    self.roomlist[i].setPraesenz(True)

        elif data[0] == 62:
            for i in range(len(self.rooms)):
                if self.rooms[i][0] == station:
                    self.roomlist[i].setPraesenz(False)

        elif data[0] == 41:
            for i in range(len(self.rooms)):
                if self.rooms[i][0] == station:
                    self.roomlist[i].setXX(data)

        elif data[0] == 256:
            for i in range(len(self.rooms)):
                if self.rooms[i][0] == station:
                    self.roomlist[i].setVolt( (data[1], data[2]) )

class Room(Tk.LabelFrame):

    def t_format(thigh, tlow):
        """ Formatiert den Temperaturwert und berücksichtigt dabei
            negative Werte und den Code für einen fehlenden Sensor -999/0xFC19 """
        t = thigh * 256 + tlow
        if t == 0xFC19:
            t = " --.-°"
        else:
            if t  >= 32767:
                t = -65536 + t
            t = "{:5}°".format(str(t/10))
        return t


    def __init__(self, parent, id = None, name = "Raumliste", col = 0, row = 0):
        self.callback = None
        self.id = id
        self.frm = Tk.LabelFrame(parent, text = name, height = 20)
        self.frm.grid(column = col, row = row, columnspan = 4, padx = 5, pady = 5, sticky = "nsew")
                        # Bezeichnung, init-Wert, col, row
        self.items = (  ("Messpunkt 1:",      "--.-°", 0, 0),
                        ("Messpunkt 2:",      "--.-°", 0, 1),
                        ("Messpunkt 3:",      "--.-°", 0, 2),
                        ("Raumtemperatur:",   "--.-°", 2, 0),
                        ("Luftfeuchtigkeit:", "--.-%", 2, 1),
                        ("Helligkeit:" ,      "----",  2, 2),
                        ("Präsenz:"      ,    "----",  2, 3),
                        ("U-Xbee [mV]:",      "----",  0, 3 ) )
        self.varlist = []
        for i in range(len(self.items)):
            self.varlist.append(Tk.StringVar())
            self.varlist[i].set(self.items[i][1])
            Tk.Label(self.frm, text = self.items[i][0], width = 16,
                    anchor = "e").grid(column = self.items[i][2],
                    row = self.items[i][3], sticky = "nse")
            Tk.Label(self.frm, textvariable = self.varlist[i], width = 6,
                    anchor = "w").grid( column = self.items[i][2]+ 1,
                    row = self.items[i][3], sticky = "nsw")

##        self.varlist.append(Tk.StringVar())
##        self.varlist[6].set("?")
##        Tk.Label(self.frm, text = "Präsenz:", width = 6).grid(column = 2, row = 3, sticky ="nse")
##        self.gr = Tk.Label(self.frm, textvariable = self.varlist[6], width = 6, anchor = "center")
##        self.gr.grid( column = 3, columnspan = 1, pady = 10, padx = 5,row = 3, sticky = "nsw")

        self.btn1 = Tk.Button(self.frm, text = "aktualisieren",
                    width = 12, command = self.btn1Pressed)
        self.btn1.grid(column = 2, columnspan = 2,row = 4, padx = 10, pady = 5, sticky = "nsew")

        self.btn2 = Tk.Button(self.frm, text = "U-Xbee messen",
                    width = 12, command = self.btn2Pressed)
        self.btn2.grid(column = 0, columnspan = 2,row = 4, padx = 10, pady = 5, sticky = "nsew")


        self.frm2 = Tk.LabelFrame(self.frm, text = "Schalter")
        self.frm2.grid(column = 0, row = 5, columnspan = 4, padx = 5, pady = 5, sticky = "nsew")
                        # Name, col, row
        self.checkbtns = (  ("Schalter 1", 0, 0),
                            ("Schalter 2", 1, 0),
                            ("Schalter 3", 2, 0),
                            ("Schalter 4", 3, 0) )

        self.checkbtnlist = []
        for i in range(len(self.checkbtns)):
            self.checkbtnlist.append(Tk.IntVar())
            Tk.Checkbutton( self.frm2, text = self.checkbtns[i][0],
                            variable = self.checkbtnlist[i], width = 7,
                            command = self.checkbtncmd, anchor = "w").grid(
                            column  = self.checkbtns[i][1],
                            row  = self.checkbtns[i][2],
                            padx = 5, pady = 5,
                            sticky = "nsew" )


    def checkbtncmd(self):
        states = []
        for i in self.checkbtnlist:
            states.append(i.get())
        print(states)


    def setCallback(self, callback):
        self.callback = callback


    def setLight(self, light):
        """ Schreibt den Helligkeitswert ins Window """
        self.varlist[5].set(str(light))


    def setPraesenz(self, val):
        """ Bewegungs/Präsenzmelder """
        if val:
            txt = "YES"
##            self.gr.configure( bg = 'red')
        else:
            txt = "NO"
##            self.gr.configure( bg = 'green')
        self.varlist[6].set(txt)


    def setXX(self, data):
        """ Testfunktion, wertet das erste Byte aus:
            hier ist der Datensatztyp codiert.
            Danach können die Daten interpretiert und in die StringVars
            kopiert werden.
        """
        self.setLight(data[5] * 256 + data[6])
        rh = data[1] * 256 + data[1]

        if rh == 0:
            rh = " --.-%"
            th = " --.-°"
        else:
            rh = "{:5}%".format(rh/10)
            th = Room.t_format(data[3], data[4])

        self.varlist[4].set(rh)
        self.varlist[3].set(th)

        j = 0
        for i in range(7, len(data), 2):
            t = Room.t_format(data[i], data[i+1])
            self.varlist[j].set(t)
            j += 1


    def btn1Pressed(self):
        """ wird beim Betätigen von Btn1 aufgerufen,
            reicht die RaumId und einen Code für die auszulösende Funktion
            weiter:
            hier DataFrame mit den Daten 0x54, 0xFF """

        #print("btn1Pressed()", self.callback)
        if self.callback is not None:
            # code 3 = sendDataFrame, data = (station, (data,))
            self.callback(3, (self.id, (0x54, 255) ) )


    def btn2Pressed(self):
        """ wird beim Betätigen von Btn1 aufgerufen,
            reicht die RaumId und einen Code für die auszulösende Funktion
            weiter:
            hier: RemoteAtCmd "%V" mit rxHandler read_U_Xbee """

        #print("btn2Pressed()", self.callback)
        if self.callback is not None:
            # code 2 = sendRemoteAtCmd, data = (atcmd, data, rxHandler)
            self.callback(2, (self.id, "%V", [], self.read_U_Xbee))


    def read_U_Xbee(self, station, data):
        if data[17] == 0:           # Frame 0x97
            if len(data) >= 20:
                self.setVolt( (data[18], data[19]) )
        else:
            print("read_U_Xbee(): Übertragungsfehler")

    def setVolt(self, data):
        """ schreibt den Wert für die Betriebsspannung """
        volt = (data[0] * 256 + data[1]) * 1200 // 1024
        self.varlist[7].set(volt)


if __name__ is "__main__":
    top = Tk.Tk()
    top.title("Reine Grafik ohne hinterlegte Funktionalität !")
    g = Gui(top)
    top.bell()
    top.mainloop()