# -*- coding: UTF-8 -*-
#       06.11.2013
# --- [ xbee_help.py ] ---------------------------------------------------------
#
# hier werden die Frames strukturiert ausgegeben.

def x_print(byte_data):

    hex_data = ""
    for i in byte_data:
        hex_data += "{:>02X}".format(i)

    frm_typ = byte_data[3]

    # Nun den Frametyp herausdestillieren und zur zugehörigen Ausgabe springen
    if frm_typ == 0x88:
        __frm88(hex_data)

    elif frm_typ == 0x8A:
        __frm8A(hex_data)

    elif frm_typ == 0x8B:
        __frm8B(hex_data)

    elif frm_typ == 0x90:
        __frm90(hex_data)

    elif frm_typ == 0x91:
        __frmXX(hex_data)

    elif frm_typ == 0x92:
         __frm92(hex_data)

    elif frm_typ == 0x94:
        __frmXX(hex_data)

    elif frm_typ == 0x95:
        __frm95(hex_data)

    elif frm_typ == 0x97:
        __frm97(hex_data)

    elif frm_typ == 0xA0:
        __frmXX(hex_data)

    elif frm_typ == 0xA1:
        __frmA1(hex_data)

    elif frm_typ == 0xA3:
        __frmA3(hex_data)

    else:
        raise ValueError("Unbekannter Frametyp :" + str(frm_data[0]))

# --- FRAME 0x88 ---------------------------------------------------------------

frm88_err = ("OK", "Error", "Invalid Command", "Invalid Parameter", "Tx Failure")

def __frm88(data):
    """ Das Frame 0x88 ist die Rückmeldung auf ein lokales AT-Command"""
    if len(data) > 16:
        data_string = data[16:]
    else:
        data_string = ""

    idx = int(data[14:16], 16)
    if idx > len(frm88_err):
                 raise IndexError("Frame 88")

    msg = []
    msg.append("-> FRAME 0x88 - AT Command Response")
    msg.append("Bytes         : " + data)
    msg.append("Frame ID      : " + data[8:10])
    msg.append("AT-Command    : " + (chr(int(data[10:12], 16)) + chr(int(data[12:14], 16))))
    msg.append("Cmd Status    : " + data[14:16] + '-' + frm88_err[idx])

    if data_string:
        data_hex = ""
        data_dez = ""
        data_asc = ""

        for i in range(0, len(data_string), 2):
            c = int(data_string[i:i+2], 16)
            data_hex += "  " + data_string[i:i+2]
            data_dez += " " + "{:>3}".format(c)
            if c > 32 and c < 128:
                data_asc += "   " + chr(c)
            else:
                data_asc += "    "

        msg.append("Data [hex]    : " + data_hex)
        msg.append("Data [dez]    : " + data_dez)
        msg.append("Data [asc]    : " + data_asc)

    __send(msg)

# --- Frame 0x8a ---------------------------------------------------------------

frm8A_err = {
       0x00 : "Hardware Reset",
       0x01 : "Watchdog timer reset",
       0x02 : "Joined Network",
       0x03 : "Disassociated",
       0x06 : "Coordinator started",
       0x07 : "Network security key was updated",
       0x0D : "Voltage supply limit exceeded",
       0x10 : "Key establishment complete",
       0x80 : "stack error"}

def __frm8A(data):
    """ Modem Status """

    msg = []
    msg.append("-> FRAME 0x8A - Modem Status")
    msg.append("Bytes         : " + data)
    msg.append("Frm Status    : " + data[8:10] + "-"+ frm8A_err.get(int(data[8:10],16), "?"))

    __send(msg)

# -- Frame 0x8B ----------------------------------------------------------------

delivery = {0x00 : "Success",
            0x01 : "MAC ACK Failure",
            0x02 : "CCA Failure",
            0x15 : "Invalid destination endpoint",
            0x21 : "Network ACK Failure",
            0x22 : "Not Joined to Network",
            0x23 : "Self adressed",
            0x24 : "Adress not found",
            0x25 : "Route not found",
            0x26 : "Broadcast source failed to hear a neighbor relay",
            0x2B : "Invalid binding table index",
            0x2C : "Resource error lack of ...",
            0x32 : "Resource error lack of ...",
            0x74 : "Data payload too large",
            0x75 : "Indirect message unrequested"
            }

discovery = {0x00 : "No Discovery Overhead",
             0x01 : "Address Discovery",
             0x02 : "Route Discovery",
             0x03 : "Adress and Route",
             0x40 : "Extended Timeout Discovery"
             }

def __frm8B(data):
    """ ZigBee Transmit Status """

    msg = []
    msg.append("-> FRAME 0x8B - ZigBee Transmit Status")
    msg.append("Bytes         : " + data)
    msg.append("Frame ID      : " + data[8:10])
    msg.append("Destin. 16-Bit: " + data[10:14])
    msg.append("Tx Retry Cnt  : " + data[14:16])
    msg.append("Deliv. Status : " + data[16:18] + "-" + delivery.get(int(data[16:18],16), "?"))
    status = int(data[18:20],16)
    if status == 0:
        discovery_str = []
        discovery_str.append(discovery.get(0))
    else:
        discovery_str = __get_bits_in_dict(discovery, status)

    msg.append("Discov.Status : " + data[18:]   + "-" + discovery_str[0])
    i = int(data[18:19] + "0", 16)
    if i in range(1, len(discovery_str)):
        msg.append("              : " + data[18:] + "-" + discovery_str[i])

    __send(msg)

# --- Frame 0x90 ---------------------------------------------------------------

# Rx-Options gelten für Frm0x90, Frm0x91, Frm0x92, Frm0x95, Frm0xA0, Frm0xA1
frm_rx_options = {
    0x01 : "Packet Ack",
    0x02 : "Broadcast Packet",
    0x20 : "Packet encrypted",
    0x40 : "Send from end device"}

def __frm90(data):
    """ Frame 0x90 liefert die Daten aus, das eine andere Station via Frame0x10 gesendet hat """

    msg = []
    msg.append("-> FRAME 0x90 - ZigBee Receive Packet")
    msg.append("Bytes         : " + data)
    msg.append("Source 64-Bit : " + data[8:24])
    msg.append("Source 16-Bit : " + data[24:28])
    options = __get_bits_in_dict(frm_rx_options, int(data[29:30],16))
    msg.append("Rx Options    : " + data[28:30] + "-" + options[0])
    for i in range(1, len(options)):
        msg.append("             : " + data[28:30] + "-" + options[i])

    data_string = data[30:]

    if data_string:
        data_hex = ""
        data_dez = ""
        data_asc = ""

        for i in range(0, len(data_string), 2):
            c = int(data_string[i:i+2], 16)
            data_hex += "  " + data_string[i:i+2]
            data_dez += " " + "{:>3}".format(c)
            if c > 32 and c < 128:
                data_asc += "   " + chr(c)
            else:
                data_asc += "    "

        msg.append("Data [hex]    : " + data_hex)
        msg.append("Data [dez]    : " + data_dez)
        msg.append("Data [asc]    : " + data_asc)

    __send(msg)

# --- ZigBee Expicit Rx Indicator
##def frm91(time, data):
##    cmd_data = ""                       # die NutzDaten im Frame
##    for i in range(1, len(data) - 1):
##        cmd_data += f_hex(data[i]) + ','
##
##
##    t = "> {} - Frm: {} data: ".format(time, data[0], cmd_data)
##    tx.insert('end', t +"\n")
##    tx.see('end')


# --- ZigBee IO data Sample Rx Indicator

frm92_options = { 0x01 : "Packet Acknowledged",
                  0x02 : "Packet was a broadcast packet"}

def __frm92(data):

    msg = []
    msg.append("-> FRAME 0x92 - ZigBee IO Data Sample Rx Indicator")
    msg.append("Bytes         : " + data)
    msg.append("Source 64-Bit : " + data[8:24])
    msg.append("Source 16-Bit : " + data[24:28])
    options = __get_bits_in_dict(frm_rx_options, int(data[28:30], 16))

    msg.append("Rx Options    : " + data[28:30] + "-" + options[0])
    for i in range(1, len(options)):
        msg.append("              : " + data[28:30] + "-" + options[i])

    # Samples steht immer auf 1, die Ausgabe enhält daher keine Information
    #msg.append("Samples       : " + data[30:32] + " always 1")
    d_mask = int(data[32:36],16)

    if d_mask:
        d_mask_bits  = " "
        d_value_bits = " "
        d_value = int(data[38:42], 16)
        for i in range(15, -1, -1):
            m = 2**i
            if d_mask & m:
                d_mask_bits += "#"
                if d_value & m:
                    d_value_bits += "1"
                else:
                    d_value_bits += "0"
            else:
                d_mask_bits  += '.'
                d_value_bits += '.'

            if i % 4 == 0:
                d_mask_bits  += " "
                d_value_bits += " "
    else:
        d_mask_bits = ""

    msg.append("DigitalCh.Mask: " + data[32:36] + d_mask_bits)
    if d_mask:
        msg.append("Digital_States: " + data[38:42] + d_value_bits)

    a_mask = int(data[36:38],16)
    if a_mask:
        a_mask_bits = "   "

        for i in range(3, -1, -1):
            m = 2**i
            if a_mask & m:
                a_mask_bits += '#'
            else:
                a_mask_bits += '.'
    else:
        a_mask_bits = ""

    msg.append("Analog Ch.Mask: " + data[36:38] + a_mask_bits)
    if d_mask:
        if a_mask:
            a_data_str = ""
            for i in range(42, len(data) -1, 4):
                a_data_str += data[i : i+4] + " "
            msg.append("Analog Samples: " + a_data_str )

    elif a_mask:
        a_data_str = ""
        for i in range(38, len(data) -1, 4):
            a_data_str += data[i : i+4] + " "

        msg.append("Analog Samples: " + a_data_str)

    __send(msg)

# --- Frame 0x95 ---- Node Identification Indictor

#frm95_option = ("", "Packet Acknowledged", "Packet was a broadcast packet")
frm95_typ    = ("Coordinator", "Router", "End Device")
frm95_event  = ("", "pushbutton event", "join event", "power cycle event")

def __frm95(data):
    """ wird empfangen, wenn sich die Gegenstelle nach einem "ND" identifiziert,
        das ist:
        - wenn der CB-Button gedrcükt wird (z.B. durch das AT-CMD "CB1")
        - sich die Gegenstelle beim Einloggen identifiziert,
        - """

    ni_string = ""
    for i in range(50, len(data), 2):
        if int(data[i:i + 2],16):
            ni_string += chr(int(data[i: i+2], 16))
        else:
            break
    ni_hex_string = data[50: i + 2]
    # i zeigt jetzt auf die den String terminierende "Null"
    i += 2 # nun zeigt i auf Parent

    parent  = data[i:i + 4]         #parent
    digi_ID = data[i + 8: i + 12]   #Digi Profile ID
    manu_ID = data[i + 12 :]        #Manufacturer ID

    msg = []
    msg.append("-> FRAME 0x95 - Node Identification Indicator")
    msg.append("Bytes         : " + data)
    msg.append("Source 64-bit : " + data[8:24])
    msg.append("Source 16-bit : " + data[24:28])

    options = __get_bits_in_dict(frm_rx_options, int(data[28:30], 16))

    msg.append("Rx Options    : " + data[28:30] + "-" + options[0])
    for t in range(1, len(options)):
        msg.append("              : " + data[28:30] + "-" + options[t])

    msg.append("Remote 16-bit : " + data[30:34])
    msg.append("Remote 64-Bit : " + data[34:50])
    msg.append("NI-String     : \"" + ni_string + "\"")
    msg.append("Parent 16-Bit : " + parent)
    idx = int(data[i + 4: i + 6])
    msg.append("Device Type   : " + data[i + 4: i + 6] + " " + frm95_typ[idx])
    idx = int(data[i + 6: i + 8], 16)
    msg.append("Source Event  : " + data[i + 6: i + 8] + " " + frm95_event[idx])
    msg.append("Digi Profile  : " + digi_ID)
    msg.append("Manufact.Prof.: " + manu_ID)

    __send(msg)

# --- Frame 0x97 ----

frm97_err = ("OK", "Error", "Invalid Cmd", "Invalid Parm", "Transmission Failed")

def __frm97(data):
    """ Das Frame 0x97 ist die Rückmeldung auf ein Remote AT-Command """

    atcmd = chr(int(data[30:32], 16)) + chr(int(data[32:34], 16))

    idx = int(data[34:36], 16)
    if idx > len(frm97_err):
                 raise IndexError("Frame 97")

    data_str = data[36:]
    msg = []
    msg.append("-> FRAME 0x97 - Remote Command Response")
    msg.append("Bytes         : " + data)
    msg.append("Frame ID      : " + data[8:10])
    msg.append("Remote 64Bit  : " + data[10:26])
    msg.append("Remote 16Bit  : " + data[26:30])
    msg.append("AT-Command    : " + atcmd)
    msg.append("Cmd Status    : " + data[34:36] + "-" + frm97_err[idx])

    if len(data_str):
        data_hex = ""
        data_dez = ""
        data_asc = ""

        for i in range(0, len(data_str), 2):
            c = int(data_str[i:i+2], 16)
            data_hex += "  " + data_str[i:i+2]
            data_dez += " " + "{:>3}".format(c)
            if c > 32 and c < 128:
                data_asc += "   " + chr(c)
            else:
                data_asc += "    "

        msg.append("Data [hex]    : " + data_hex)
        msg.append("Data [dez]    : " + data_dez)
        msg.append("Data [asc]    : " + data_asc)

    __send(msg)

# --- Over-the-air Firmware Update Status
##def frmA0(time, data):
##    cmd_data = ""                       # die NutzDaten im Frame
##    for i in range(1, len(data) - 1):
##        cmd_data += f_hex(data[i]) + ','
##
##
##    t = "> {} - Frm: {} data: ".format(time, data[0], cmd_data)
##    tx.insert('end', t +"\n")
##    tx.see('end')

# --- Route Record Indicator
def __frmA1(data):

    if len(data) < 32:
        print("Data Lenght Error 0xA1 ", data)
        return

    msg = []
    msg.append("-> FRAME 0xA1 - Route Record Indicator")
    msg.append("Bytes         : " + data)
    msg.append("Remote 64Bit  : " + data[8:24])
    msg.append("Remote 16Bit  : " + data[24:28])

    options = __get_bits_in_dict(frm_rx_options, int(data[28:30], 16))

    # "0" als rx-option gibts laut Datenblatt nicht, kommt aber vor
    # da versagt dann der Bit-Vergleich
    if len(options) == 0:
        options.append("?")

    msg.append("Rx Options    : " + data[28:30] + "-" + options[0])
    for i in range(1, len(options)):
        msg.append("              : " + data[28:30] + "-" + options[i])

    msg.append("Number of Hops: " + data[30:32])

    if int(data[30:32],16) > 0:         # Hops vorhanden
        t = data[32:]
        adr = ""
        for i in range(0, len(data), 4):
            adr += t[i: i+4] + " "
        msg.append("Adress of Hops: " + adr)

    __send(msg)

# --- Many-to-One Route Request Indicator
def __frmA3(data):

    msg = []
    msg.append("-> FRAME 0xA3 - Many-to-One Route Req.Indicator")
    msg.append("Bytes         : " + data)
    msg.append("Source 64Bit  : " + data[8:24])
    msg.append("Source 16Bit  : " + data[24:28])
    #data[29:30] ist immer "0x00"

    __send(msg)

# --- Allgemeine Formatierung
def __frmXX(data):

    msg = [data]
    __send(msg )

def __send(msg):
    for i in msg:
        print(i)
    print()

def __get_bits_in_dict(dictionary, byte):
    """ Prüft, ob in 'byte' die Bits der Keys des Dictonary's gesetzt sind.
        Wenn ja, dann wird der zugehörige String im Dictonary zur Rückgabeliste
        hinzukopiert.
    """
    result = list()
    l = list(dictionary.keys())
    for i in l:
        if i & byte:
            result.append(dictionary[i])
    return result
# --- [ eof ] ------------------------------------------------------------------
